#  Copyright (C) 2025
#  ABM, Moscow
#
#  UNPUBLISHED PROPRIETARY MATERIAL.
#  ALL RIGHTS RESERVED.
#
#  Authors: Mike Orlov <m.orlov@abm-jsc.ru>
class SecurityException(Exception):
    pass


class Unauthorized(SecurityException):
    pass


class Forbidden(SecurityException):
    pass


class AuthInfoException(SecurityException):
    pass
