#  Copyright (C) 2025
#  ABM, Moscow
#
#  UNPUBLISHED PROPRIETARY MATERIAL.
#  ALL RIGHTS RESERVED.
#
#  Authors: Mike Orlov <m.orlov@abm-jsc.ru>
from enum import StrEnum


class OAuth2Flow(StrEnum):
    implicit = "implicit"
    authorizationCode = "authorizationCode"
    password = "password"
    clientCredentials = "clientCredentials"
